import pygame
import toolbox
import random
from PodSixNet.Connection import ConnectionListener, connection
from time import sleep
import animation


running = True


class MyGameClient(ConnectionListener):
    def __init__(self, host, port):
        """
        Client constructor function. This is the code that runs once
        when a new client is made.
        """
        ConnectionListener.__init__(self)
        
        # Start the game
        pygame.init()
        pygame.mixer.pre_init(buffer=1024)
        game_width = 1000
        game_height = 650
        self.screen = pygame.display.set_mode((game_width, game_height))
        self.clock = pygame.time.Clock()

        pygame.mixer.music.load("../assets/sfx/bensound-epic.mp3")
            # -1 will keep music playing
        pygame.mixer.music.play(-1)

        # Load all the sfx for our game
        self.sfx_hit = pygame.mixer.Sound("../assets/sfx/bump.wav")
        self.sfx_swoosh = pygame.mixer.Sound("../assets/sfx/shot.wav")
        self.sfx_respawn = pygame.mixer.Sound("../assets/sfx/powerup.wav")
        self.sfx_winner = pygame.mixer.Sound("../assets/sfx/WinJingle.wav")

        self.stuffToDrawNow = []
        self.stuffToDrawNext = []

        self.background = pygame.image.load("../assets/bg_lava.png")
        #self.avatar_image = pygame.image.load("../assets/Monkey/Monkey_jump_2.png")
        self.avatar_images = [pygame.image.load("../assets/Goblin/Goblin_Avatar_1.png"),
                              pygame.image.load("../assets/Monkey/Monkey_Avatar_1.png"),
                              pygame.image.load("../assets/Ninja/Ninja_Avatar_1.png"),
                              pygame.image.load("../assets/Monkja/Monkja_Avatar_1.png"),
                              pygame.image.load("../assets/BearTrap/BearTrap_Avatar_1.png") ]
                              #pygame.image.load("../assets/Ninja/Ninja_idle_1")
        #self.fighter_image = pygame.image.load("../assets/Monkey/Monkey_idle_1.png")
        self.platform_images_base = [ pygame.image.load("../assets/platform_lava1.png"),
                                      pygame.image.load("../assets/platform_lava2.png"),
                                      pygame.image.load("../assets/platform_lava_wall1.png") ]
        self.platform_images = {}
        self.animation_managers = {}

        self.big_font = pygame.font.SysFont("default", 60, True) # opt. boolean is for bold
        self.medium_font = pygame.font.SysFont("default", 40)
        self.small_font = pygame.font.SysFont("default", 20)
        
        # Boolean in ready_text is for anti-aliasing
        self.ready_text = self.small_font.render("Ready!!", True, (0, 255, 0))
        self.not_ready_text = self.small_font.render("Not Ready", True, (255, 0, 0))

        self.connected = False

        self.Connect((host, port))


    def update(self):
        """
        Client update function. This is the function that runs over and
        over again, once every frame of the game.
        """
        connection.Pump()
        self.Pump()
        #connection.Send({"action": "hi"}) This will be sent over and over

        # Set running to False if the player clicks the X or presses esc
        global running
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
            if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                running = False

        connection.Send({"action": "keys",
                         "keys": pygame.key.get_pressed()})


        for thing in self.stuffToDrawNow:
            self.screen.blit(thing[0], thing[1])
        
            
        # Tell pygame to update the screen
        pygame.display.flip()
        self.clock.tick(30)
        pygame.display.set_caption("Python: Sprite Arena | Which Sprite will win? fps: " + str(self.clock.get_fps()))


    def ShutDown(self):
        """
        Client ShutDown function. Disconnects and closes the game.
        """
        self.connected = False
        connection.Close()
        pygame.quit()
        exit()
        
                
    #####################################
    ### Client-side Network functions ###
    #####################################
    """
    Each one of these "Network_" functions defines a command
    that the server will tell you (the client) to do.
    """
    def Network_flip(self, data):
        """
        Move all the stuff from stuffToDrawNext into stuffToDrawNow
        """
        self.stuffToDrawNow.clear()
        for thing in self.stuffToDrawNext:
            self.stuffToDrawNow.append(thing)
        self.stuffToDrawNext.clear()


    def Network_setup_platforms(self, data):
        """
        Resize the platform image to be the size of the platform
        """
        image = self.platform_images_base[data["type"]]
        squished_image = pygame.transform.scale(image, data["size"])
        self.platform_images[data["number"]] = squished_image

    def Network_setup_fighter(self, data):
        """
        Make an AnimationManager for the new fighter
        """
        new_manager = getattr(animation, data["fighter_name"]+"AnimationManager")()
                        #animation.MonkeyAnimationManager()
        self.animation_managers[data["number"]] = new_manager


    def Network_play_sfx(self, data):
        # location, ""
        getattr(self, "sfx_" + data["name"]).play()
    

    def Network_draw_background(self, data):
        self.stuffToDrawNext.append([self.background, (0, 0)])


    def Network_draw_avatar(self, data):
        avatar_image = self.avatar_images[data["fighter_type"]]
        avatar_rect = avatar_image.get_rect(center=data["coords"])
        self.stuffToDrawNext.append([avatar_image, avatar_rect])

        if data["ready"]:
            text_to_draw = self.ready_text
        else:
            text_to_draw = self.not_ready_text
        text_rect = text_to_draw.get_rect(midtop = avatar_rect.midbottom)
        text_rect.y += 4
        self.stuffToDrawNext.append([text_to_draw, text_rect])

        num_text = self.medium_font.render(str(data["number"]), True, data["color"])
        # text_rect s here will overrite previous rect
        text_rect = num_text.get_rect(midbottom=avatar_rect.midtop)
        text_rect.y -=4
        self.stuffToDrawNext.append([num_text, text_rect])

    def Network_draw_fighter(self, data):
        #image_to_draw = self.fighter_image
        image_to_draw = self.animation_managers[data["number"]].getImage(flipped=data["flipped"])
        image_rect = self.animation_managers[data["number"]].getRect(data["flipped"], data["coords"])
                    #image_to_draw.get_rect(midbottom=data["coords"])
        self.stuffToDrawNext.append([image_to_draw, image_rect])

        num_text = self.medium_font.render(str(data["number"]), True, data["color"])
        # text_rect s here will overrite previous rect
        text_rect = num_text.get_rect(midbottom=image_rect.midtop)
        text_rect.y -=4
        self.stuffToDrawNext.append([num_text, text_rect])

    def Network_draw_platform(self, data):
        #surface = pygame.Surface(data["size"])
        #surface.fill((50, 0, 75))
        #self.stuffToDrawNext.append([surface, data["coords"]])
        x, y = data["coords"]
        # The ''y -= __'' number may be different for different platforms
        y -= 30
        self.stuffToDrawNext.append([self.platform_images[data["number"]], (x, y)])


    def Network_draw_hitbox(self, data):
        surface = pygame.Surface(data["size"])
        surface.fill((50, 0, 75))
        self.stuffToDrawNext.append([surface, data["coords"]])


    def Network_draw_percentage(self, data):
        # The int would chop of the decimals of a number
        # So multiplying by 100 would keep all the number's characters
        percentage_string = str(int(data["percentage"]*100)) + "%"
        # Boolean is for anti-aliasing
        text = self.medium_font.render(percentage_string, True, data["color"])
        text_rect = text.get_rect(midbottom=data["coords"])
        self.stuffToDrawNext.append([text, text_rect])

        icon = self.animation_managers[data["number"]].icon
        x, y = text_rect.bottomleft
        y += 2
        for i in range(data["lives"]): # Number inside () will happen that often
            self.stuffToDrawNext.append([icon, (x,y)])
            x += icon.get_width() + 2

            
    def Network_draw_victory(self, data):
        image = self.animation_managers[data["number"]].getImage()
        center_screen = (self.screen.get_width()/2, self.screen.get_height()/2)
        image_rect = image.get_rect(center=center_screen)
        self.stuffToDrawNext.append([image, image_rect])

        message = "Player " + str(data["number"]) + " WINS!!!"
        # boolean is for anti-aliasing
        text = self.big_font.render(message, True, data["color"])
        text_rect = text.get_rect(center=center_screen)
        text_rect.y -= 120
        self.stuffToDrawNext.append([text, text_rect])

        if data["time_left"] <= 10:
            # message below will overrite old one above
            # int will make "time_left" an integer (cutting of the decimals)
            message = "Returning to lobby in " + str(int(data["time_left"])) + " seconds"
            # last parameter for font render below is color
            text = self.medium_font.render(message, True, data["color"])
            text_rect = text.get_rect(center=center_screen)
            text_rect.y += 120
            self.stuffToDrawNext.append([text, text_rect])


    def Network_change_fighter_state(self, data):
        self.animation_managers[data["number"]].ChangeState(data["state"])
        

    def Network_connected(self, data):
        """
        Network_connected runs when you successfully connect to the server
        """
        self.connected = True
        print("Connection succesful!")
        
    
    def Network_error(self, data):
        """
        Network_error runs when there is a server error
        """
        print('error:', data['error'][1])
        self.ShutDown()
    
    def Network_disconnected(self, data):
        """
        Network_disconnected runs when you disconnect from the server
        """
        print('Server disconnected')
        self.ShutDown()

# Remember to change this line if you connetct from a different computer
# "...." comment out input and replace with this to remember host ip
ip = input("Please enter the host's IP address: ")
port = 5555
thisClient = MyGameClient(ip, port)

"""This is the loop that keeps going until the game is closed"""
while running:
    thisClient.update()
    
thisClient.ShutDown()
