import pygame
import toolbox

from hitbox import Hitbox

class Fighter(pygame.sprite.Sprite):
    def __init__(self, spawn_coords, number):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.spawn_coords = spawn_coords
        self.number = number
        self.image = pygame.image.load("../assets/Monkey/Monkey_idle_1.png")
        self.rect = self.image.get_rect(midbottom=self.spawn_coords)

        self.anim_state = "idle"
        self.anim_state_changed = False

        self.sfx_to_play = None

        self.just_started_attack = False

        # X variables to mess with here
        self.run_speed = 8
        self.x_knockback_slowdown = 0.5
        # Other X variables
        self.x_move = 0
        self.x_flipped = False
        self.x_knockback_momentum = 0

        # Y variables to mess with here
        self.gravity = 1.6
        self.max_fall_speed = 30
        self.jump_speed = -24
        self.max_jumps = 3
        self.fast_fall_speed = 0.9
        # Other Y variables
        self.y_move = 0
        self.jumps_left = self.max_jumps
        self.grounded = False
        self.down_pressed = False

        self.damage_percentage = 0
        self.border_padding = 200
        self.lives = 3
        self.defeated = False

        self.jump_key = toolbox.keyDownListener()
        self.attack1_key = toolbox.keyDownListener()
        self.attack2_key = toolbox.keyDownListener()
        self.attack3_key = toolbox.keyDownListener()
        

    def checkIfOffScreen(self, game_width, game_height):
        # parentheses () will allow us to ''indent'' the same line without errors
        if (self.rect.right < 0 - self.border_padding
            or self.rect.left > game_width + self.border_padding
            or self.rect.bottom < 0 - self.border_padding
            or self.rect.top > game_height + self.border_padding):
            self.respawn()


    def respawn(self):
        self.lives -= 1
        if self.lives > 0: # OR add an = after >
            self.rect.midbottom = self.spawn_coords
            self.x_move = 0
            self.y_move = 0
            self.x_knockback_momentum = 0
            self.damage_percentage = 0
            self.sfx_to_play = "respawn"
        else:
            self.defeated = True


    def HandleInput(self, keys):
        # Update all key listeners
        self.jump_key.update(keys[pygame.K_UP])
        self.attack1_key.update(keys[pygame.K_q])
        self.attack2_key.update(keys[pygame.K_w])
        self.attack3_key.update(keys[pygame.K_e])

        self.anim_state_changed = False

        # Move left
        if keys[pygame.K_LEFT]:
            if not self.x_flipped:
                self.x_flipped = True
            else:
                self.x_move = -self.run_speed
                if self.grounded and self.anim_state != "run":
                    self.anim_state = "run"
                    self.anim_state_changed = True
        # Move right
        if keys[pygame.K_RIGHT]:
            if self.x_flipped:
                self.x_flipped = False
            else:
                self.x_move = self.run_speed
                if self.grounded and self.anim_state != "run":
                    self.anim_state = "run"
                    self.anim_state_changed = True
        # Stop moving if not pressing left or right
        if not keys[pygame.K_LEFT] and not keys[pygame.K_RIGHT]:
            self.x_move = 0
            should_idle = (self.anim_state != "idle"
                           and self.anim_state != "attack1"
                           and self.anim_state != "attack2"
                           and self.anim_state != "attack3"
                           and self.grounded)
            if should_idle:
                self.anim_state = "idle"
                self.anim_state_changed = True

        # Drop down
        if keys[pygame.K_DOWN]:
            self.down_pressed = True
            self.y_move += self.fast_fall_speed
        else:
            self.down_pressed = False
                
        # Jump
        if self.jump_key.down:
            if self.jumps_left > 0:
                self.y_move = self.jump_speed
                self.jumps_left -= 1
                self.anim_state = "jump"
                self.anim_state_changed = True
        # Infinite jump cheat lol (And-E put lol, not me)
        if keys[pygame.K_a] and keys[pygame.K_n] and keys[pygame.K_d] and keys[pygame.K_y]:
            self.max_jumps = 99999
            self.jumps_left = self.max_jumps
        # Attack 1
        if self.attack1_key.down:
            self.doAttack1()
            self.anim_state_changed = True
            self.anim_state = "attack1"
            self.just_started_attack = True
        # Attack 2
        elif self.attack2_key.down:
            self.doAttack2()
            self.anim_state_changed = True
            self.anim_state = "attack2"
            self.just_started_attack = True
        # Attack 3
        elif self.attack3_key.down:
            self.doAttack3()
            self.anim_state_changed = True
            self.anim_state = "attack3"
            self.just_started_attack = True

# in doAttack s, may need to change duration & adjust the knockback, damage
    def doAttack1(self): # high attack
        size = (30,30)
        if self.x_flipped:
            x, y = self.rect.topleft
            x -= 20
        else:
            x, y = self.rect.topright
            x += 20
        y += 25
        damage = 0.03
        # knockback--- (x,y)
        knockback = (2,15)
        duration = 2
        # creator_num, coords, size, damage, knockback, duration, flipped
        Hitbox(self.number, (x,y), size, damage, knockback, duration, self.x_flipped)

    def doAttack2(self): # mid attack
        size = (32,18)
        if self.x_flipped:
            x, y = self.rect.topleft
        else:
            x, y = self.rect.topright
        y += 65
        damage = 0.02
        # knockback--- (x,y)
        knockback = (15,2)
        duration = 2
        # creator_num, coords, size, damage, knockback, duration, flipped
        Hitbox(self.number, (x,y), size, damage, knockback, duration, self.x_flipped)

    def doAttack3(self): # low attack
        size = (42,25)
        if self.x_flipped:
            x, y = self.rect.topleft
        else:
            x, y = self.rect.topright
        y += 95
        damage = 0.04
        # knockback--- (x,y)
        knockback = (6,7)
        duration = 2
        # creator_num, coords, size, damage, knockback, duration, flipped
        Hitbox(self.number, (x,y), size, damage, knockback, duration, self.x_flipped)


    def GetHit (self, damage, knockback, flipped):
        self.sfx_to_play = "hit"
        
        self.damage_percentage += damage
        
        self.y_move -= knockback[1] * self.damage_percentage

        if flipped:
            self.x_knockback_momentum -= knockback[0] * self.damage_percentage
        else:
            self.x_knockback_momentum += knockback[0] * self.damage_percentage
            

    def update(self, platforms, hitboxes):
        if not self.defeated:
            self.sfx_to_play = None

            if self.just_started_attack:
                self.sfx_to_play = "swoosh"
                self.just_started_attack = False
            
            # Do Gravity
            self.y_move += self.gravity
            if self.y_move > self.max_fall_speed:
                self.y_move = self.max_fall_speed

            # Make X knockback momentum slow down
            if self.x_knockback_momentum > 0:
                self.x_knockback_momentum -= self.x_knockback_slowdown
            else:
                self.x_knockback_momentum += self.x_knockback_slowdown
            if abs(self.x_knockback_momentum) <= self.x_knockback_slowdown:
                self.x_knockback_momentum = 0

            
            # Do X direction movement with collision checking
            self.rect.x += self.x_move + self.x_knockback_momentum
            for platform in platforms:
                if self.rect.colliderect(platform) and not platform.jump_through:
                    # 'Hop' up corner
                    if self.rect.top < platform.rect.top:
                        self.rect.bottom = platform.rect.top
                    else:
                        if self.x_move + self.x_knockback_momentum > 0:
                            self.rect.right = platform.rect.left
                            self.x_knockback_momentum = 0
                        else:
                            self.rect.left = platform.rect.right
                            self.x_knockback_momentum = 0
                            
            # Do Y direction movement with collision checking
            self.rect.y += self.y_move
            self.grounded = False
            for platform in platforms:
                if self.rect.colliderect(platform.rect):
                    if self.y_move >= 0:
                        should_drop_through = (self.down_pressed and platform.jump_through)
                        if not should_drop_through:
                            if self.rect.top < platform.rect.top:
                                self.rect.bottom = platform.rect.top
                                self.y_move = 0
                                self.jumps_left = self.max_jumps
                                self.grounded = True
                    else:
                        if not platform.jump_through:
                            self.rect.top = platform.rect.bottom
                            self.y_move = 0
                        
            # Do hitbox collision checking
            for hitbox in hitboxes:
                if self.rect.colliderect(hitbox.rect):
                    if self.number != hitbox.creator_num:
                        self.GetHit(hitbox.damage, hitbox.knockback, hitbox.flipped)

class Goblin(Fighter):
    def __init__(self, spawn_coords, number):
        Fighter.__init__(self, spawn_coords, number)
        self.name = "Goblin"
        # X variables to mess with here
        self.run_speed = 6
        self.x_knockback_slowdown = 0.5
        # Y variables to mess with here
        self.gravity = 1.6
        self.max_fall_speed = 30
        self.jump_speed = -24
        self.max_jumps = 3
        self.fast_fall_speed = 0.9

    def doAttack1(self): # high attack
        size = (30,30)
        if self.x_flipped:
            x, y = self.rect.topleft
            x -= 20
        else:
            x, y = self.rect.topright
            x += 20
        y += 25
        damage = 0.03
        # knockback--- (x,y)
        knockback = (2,15)
        duration = 2
        # creator_num, coords, size, damage, knockback, duration, flipped
        Hitbox(self.number, (x,y), size, damage, knockback, duration, self.x_flipped)

    def doAttack2(self): # mid attack
        size = (32,18)
        if self.x_flipped:
            x, y = self.rect.topleft
        else:
            x, y = self.rect.topright
        y += 65
        damage = 0.02
        # knockback--- (x,y)
        knockback = (15,2)
        duration = 2
        # creator_num, coords, size, damage, knockback, duration, flipped
        Hitbox(self.number, (x,y), size, damage, knockback, duration, self.x_flipped)

    def doAttack3(self): # low attack
        size = (42,25)
        if self.x_flipped:
            x, y = self.rect.topleft
        else:
            x, y = self.rect.topright
        y += 95
        damage = 0.04
        # knockback--- (x,y)
        knockback = (6,7)
        duration = 2
        # creator_num, coords, size, damage, knockback, duration, flipped
        Hitbox(self.number, (x,y), size, damage, knockback, duration, self.x_flipped)



class Monkey(Fighter):
    def __init__(self, spawn_coords, number):
        Fighter.__init__(self, spawn_coords, number)
        self.name = "Monkey"
        # X variables to mess with here
        self.run_speed = 8
        self.x_knockback_slowdown = 0.5
        # Y variables to mess with here
        self.gravity = 1.6
        self.max_fall_speed = 30
        self.jump_speed = -18 # And-E did -20
            # higher jump_speed will make jumps smaller
        self.max_jumps = 10
        self.fast_fall_speed = 0.9

    def doAttack1(self): # high attack
        size = (20,15)
        if self.x_flipped:
            x, y = self.rect.topleft
        else:
            x, y = self.rect.topright
        y += 25
        damage = 0.03
        # knockback--- (x,y)
        knockback = (2,15)
        duration = 2
        # creator_num, coords, size, damage, knockback, duration, flipped
        Hitbox(self.number, (x,y), size, damage, knockback, duration, self.x_flipped)

    def doAttack2(self): # mid attack
        size = (18,18)
        if self.x_flipped:
            x, y = self.rect.topleft
        else:
            x, y = self.rect.topright
        y += 60
        damage = 0.02
        # knockback--- (x,y)
        knockback = (15,2)
        duration = 2
        # creator_num, coords, size, damage, knockback, duration, flipped
        Hitbox(self.number, (x,y), size, damage, knockback, duration, self.x_flipped)

    def doAttack3(self): # low attack
        size = (24,12)
        if self.x_flipped:
            x, y = self.rect.topleft
        else:
            x, y = self.rect.topright
        y += 90
        damage = 0.05
        # knockback--- (x,y)
        knockback = (6,7)
        duration = 2
        # creator_num, coords, size, damage, knockback, duration, flipped
        Hitbox(self.number, (x,y), size, damage, knockback, duration, self.x_flipped)


class Ninja(Fighter):
    def __init__(self, spawn_coords, number):
        Fighter.__init__(self, spawn_coords, number)
        self.name = "Ninja"
        # X variables to mess with here
        self.run_speed = 9
        self.x_knockback_slowdown = 0.5
        # Y variables to mess with here
        self.gravity = 1.6
        self.max_fall_speed = 30
        self.jump_speed = -24 # And-E did -20
            # higher jump_speed will make jumps smaller
        self.max_jumps = 4
        self.fast_fall_speed = 0.9

    def doAttack1(self): # high attack
        size = (30,20)
        if self.x_flipped:
            x, y = self.rect.topleft
            x -= 10
        else:
            x, y = self.rect.topright
            x += 10
        y += 20
        damage = 0.03
        # knockback--- (x,y)
        knockback = (2,15)
        duration = 2
        # creator_num, coords, size, damage, knockback, duration, flipped
        Hitbox(self.number, (x,y), size, damage, knockback, duration, self.x_flipped)

    def doAttack2(self): # mid attack
        size = (40,10)
        if self.x_flipped:
            x, y = self.rect.topleft
        else:
            x, y = self.rect.topright
        y += 56
        damage = 0.02
        # knockback--- (x,y)
        knockback = (15,2)
        duration = 2
        # creator_num, coords, size, damage, knockback, duration, flipped
        Hitbox(self.number, (x,y), size, damage, knockback, duration, self.x_flipped)

    def doAttack3(self): # low attack
        size = (35,25)
        if self.x_flipped:
            x, y = self.rect.topleft
        else:
            x, y = self.rect.topright
        y += 75
        damage = 0.04
        # knockback--- (x,y)
        knockback = (6,7)
        duration = 2
        # creator_num, coords, size, damage, knockback, duration, flipped
        Hitbox(self.number, (x,y), size, damage, knockback, duration, self.x_flipped)
"""
Needed for new fighters: an animation_manager, a fighter class,
AssignFighter in server, change num_of_fighters in lobbyAvatar,
load a png in self.avatar_images in client
"""
                    
